@@ -281,7 +281,7 @@ CSN_DESCR_END         (Content_t)
 
 static gint16 Content_Dissector(csnStream_t* ar, struct bitvec *vector, unsigned *readIndex, void* data)
 {
-  if (ar->direction == 0)
+  if (ar->direction == CSN_DIRECTION_ENC)
     {
       return osmo_csn1_stream_encode(ar, CSNDESCR(Content_t), vector, readIndex, data);
     }
@@ -305,7 +305,7 @@ CSN_DESCR_END         (Additional_access_technologies_t)
 
 static gint16 Additional_access_technologies_Dissector(csnStream_t* ar, struct bitvec *vector, unsigned *readIndex, void* data)
 {
-  if (ar->direction == 0)
+  if (ar->direction == CSN_DIRECTION_ENC)
   {
     return osmo_csn1_stream_encode(ar, CSNDESCR(Additional_access_technologies_t), vector, readIndex, data);
   }
